#include "krige_tileov.h"

void krige_tileov(double *xsw,
		  double *ysw,
		  double *xne,
		  double *yne,
		  double *angle,
		  int *nx,
		  int *nall,                 
		  double *dx,
		  double *dy,
		  int *iall,
		  double *xg,
		  double *yg,
		  double *zg,
		  double *varg,
		  int *dog,
		  double *lon,
		  double *lat,
		  double *z,
		  int *extrap,
		  int *n,
		  int *covtype,
		  double *covpar,
		  int *trend,
		  int *ntrend,
		  double *rsearch,
		  int *nsearch,
		  int *nsmin,
		  int *nsmax,
		  int *mode,
                  int *bits,
	          int *ierr,
		  int *nxper,
		  int *nyper
		  ){


    int nz=(*nx)*(nall[0]), ldcov=(*n), ldc0=(*n), extcov=0,
	ldfwrk=(*n), ldlmbd=(*n), nkwork=(*n+*ntrend), nt=((nall[1])*(nall[2])), ipt=((iall[0])*(iall[1]));

    double *covmat, *c0vec, *fwork, *f0work, *dist, *kwork,
	   *rhswork, *mu, *lambda, cov0, *xgwork, *ygwork, *x0, *y0, *z0, 
	   *var0;
    int *indsnb, *indsna, *indsrt, *ipiv, *do0, *inddo, *cnt;

#ifdef TRANSIENT
    covmat=Calloc((size_t)(*n)*(*n),double);
    c0vec=Calloc((size_t)(*n)*ipt,double);    
    fwork=Calloc((size_t)(*n)*(*ntrend),double);    
    f0work=Calloc((size_t)(*ntrend)*ipt,double);    
    dist=Calloc((size_t)(*n),double);    
    indsnb=Calloc((size_t)(*n),int);    
    indsna=Calloc((size_t)(*n),int);    
    indsrt=Calloc((size_t)(*n),int); 
    kwork=Calloc((size_t)(*n+*ntrend)*(*n+*ntrend),double); 
    rhswork=Calloc((size_t)(*n+*ntrend)*ipt,double);
    ipiv=Calloc((size_t)(*n+*ntrend)*ipt,int);
    mu=Calloc((size_t)(*ntrend)*ipt,double);
    lambda=Calloc((size_t)(*n)*ipt,double); 
    xgwork=Calloc((size_t)(nz),double);
    ygwork=Calloc((size_t)(nz),double); 
    x0=Calloc((size_t)(ipt),double); 
    y0=Calloc((size_t)(ipt),double); 
    z0=Calloc((size_t)(ipt),double); 
    do0=Calloc((size_t)(ipt),int); 
    inddo=Calloc((size_t)(ipt),int); 
    var0=Calloc((size_t)(ipt),double); 
    cnt=Calloc((size_t)(nz),int); 

#else
    covmat=(double *) R_alloc((*n)*(*n),sizeof(double));
    c0vec=(double *) R_alloc((*n)*ipt,sizeof(double));
    fwork=(double *) R_alloc((*n)*(*ntrend),sizeof(double));    
    f0work=(double *) R_alloc((*ntrend)*ipt,sizeof(double));    
    dist=(double *) R_alloc((*n),sizeof(double));
    indsnb=(int *) R_alloc((*n),sizeof(int));
    indsna=(int *) R_alloc((*n),sizeof(int));
    indsrt=(int *) R_alloc((*n),sizeof(int));
    kwork=(double *) R_alloc((*n+*ntrend)*(*n+*ntrend),sizeof(double)); 
    rhswork=(double *) R_alloc((*n+*ntrend)*ipt,sizeof(double)); 
    ipiv=(int *) R_alloc((*n+*ntrend)*ipt,sizeof(int)); 
    mu=(double *) R_alloc((*ntrend)*ipt,sizeof(double));
    lambda=(double *) R_alloc((*n)*ipt,sizeof(double));
    xgwork=(double *) R_alloc((nz),sizeof(double));
    ygwork=(double *) R_alloc((nz),sizeof(double)); 
    x0=(double *) R_alloc((ipt),sizeof(double)); 
    y0=(double *) R_alloc((ipt),sizeof(double)); 
    z0=(double *) R_alloc((ipt),sizeof(double)); 
    do0=(int *) R_alloc((ipt),sizeof(int)); 
    inddo=(int *) R_alloc((ipt),sizeof(int)); 
    var0=(double *) R_alloc((ipt),sizeof(double)); 
    cnt=(int *) R_alloc((nz),sizeof(int));    
#endif

    
    F77_CALL(krgtilov)(xsw, 
		     ysw, 
		     xne, 
		     yne, 
		     angle, 
		     nx, 
		     nall, 
		     &nz,
		     &nt,
		     dx, 
		     dy,
		     iall,
		     &ipt,
		     xg, 
		     yg, 
		     zg, 
		     varg, 
		     dog,
		     xgwork,
		     ygwork,
		     lon, 
		     lat, 
		     z, 
		     extrap, 
		     n, 
		     covtype, 
		     covpar, 
		     covmat, 
		     &ldcov, 
		     c0vec, 
		     &ldc0, 
		     &cov0, 
		     &extcov, 
		     trend, 
		     ntrend, 
		     rsearch, 
		     nsearch, 
		     nsmin, 
		     nsmax, 
		     fwork, 
		     &ldfwrk, 
		     f0work, 
		     dist, 
		     indsnb, 
		     indsna, 
		     indsrt, 
		     kwork, 
		     &nkwork, 
		     rhswork, 
		     ipiv, 
		     mode, 
		     mu, 
		     lambda, 
		     &ldlmbd,
		     x0,
		     y0,
		     z0,
		     do0,
		     inddo,
		     var0,
		     bits, 
		     ierr,
		     nxper,
		     nyper,
		     cnt  );
    

#ifdef TRANSIENT
    Free(cnt);
    Free(var0);
    Free(inddo);
    Free(do0);
    Free(z0);
    Free(y0);
    Free(x0);
    Free(ygwork);
    Free(xgwork);
    Free(lambda);
    Free(mu);
    Free(ipiv);
    Free(rhswork);
    Free(kwork);
    Free(indsrt);
    Free(indsna);
    Free(indsnb);
    Free(dist);
    Free(f0work);
    Free(fwork);
    Free(c0vec);
    Free(covmat);
#endif
}
    
